package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.utils.PropertiesUtils;
import cn.com.ctop.toutiao.common.BytedanceInterfaceConstant;
import com.alibaba.druid.support.spring.stat.annotation.Stat;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.ResultMapUtils;
import org.jeecg.common.util.StatusCode;
import org.jeecg.modules.ctop.entity.*;
import org.jeecg.modules.ctop.mapper.ByteDanceImageInfoMapper;
import org.jeecg.modules.ctop.mapper.ByteDanceVideoInfoMapper;
import org.jeecg.modules.ctop.mapper.FileInfoMapper;
import org.jeecg.modules.ctop.service.IByteDanceImageInfoService;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.jeecg.modules.ctop.service.IFileInfoService;
import org.jeecg.modules.system.entity.SysCategory;
import org.jeecg.modules.system.mapper.SysCategoryMapper;
import org.jeecg.modules.system.service.ISysCategoryService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.URI;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 文件信息
 * @Author: jeecg-boot
 * @Date: 2019-07-28
 * @Version: V1.0
 */
@Service
public class FileInfoServiceImpl extends ServiceImpl<FileInfoMapper, FileInfo> implements IFileInfoService {
    private static final Logger logger = LoggerFactory.getLogger(FileInfoServiceImpl.class);

    @Override
    public Map<String, Object> uploadVideoToBytedance(String accountId, String videoUrl) {
        Map<String, Object> resultMap = new HashMap<>();
        //TODO查询是否已经上传过头条平台
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject resultObject = uploadAdvideo(token.getAccessToken(), videoUrl, token.getAccountId() + "");
        System.out.println(resultObject);
        Integer code = resultObject.getInteger("code");
        String message = resultObject.getString("message");
        if (null == code || code != 0) {
            logger.info("视频文件上传失败,accountId:{},message:{}", accountId, message);
            ResultMapUtils.setResultMap(resultMap, StatusCode.BYTEDANCE_VIDEO_UPLOAD_FAIL.getCode());
            return resultMap;
        }
        JSONObject data = resultObject.getJSONObject("data");
        ByteDanceVideoInfo videoInfo = new ByteDanceVideoInfo(data, token);
        videoInfoMapper.insert(videoInfo);
        ResultMapUtils.setResultMap(resultMap, StatusCode.COMMON_SUCCESS.getCode());
        resultMap.put("videoId", videoInfo.getId());
        return resultMap;
    }

    @Override
    public Map<String, Object> upload3ImagesToBytedance(String accountId, String imageFileUrls) {
        Map<String, Object> resultMap = new HashMap<>();
        String[] imageUrls = imageFileUrls.split(",");
        if (null == imageUrls || imageUrls.length < 3) {
            ResultMapUtils.setResultMap(resultMap, StatusCode.IMAGE_NUMBER_SHORTAGE.getCode());
            return resultMap;
        }
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONArray imageIds = new JSONArray();
        for (int i = 0; i < imageUrls.length; i++) {
            JSONObject resultObject = uploadAdImage(token.getAccessToken(), imageUrls[i], token.getAccountId() + "");
            System.out.println(resultObject);
            Integer code = resultObject.getInteger("code");
            String message = resultObject.getString("message");
            if (null == code || code != 0) {
                logger.info("图片文件上传失败,accountId:{},message:{}", accountId, message);
                ResultMapUtils.setResultMap(resultMap, StatusCode.BYTEDANCE_VIDEO_UPLOAD_FAIL.getCode());
                return resultMap;
            }
            JSONObject data = resultObject.getJSONObject("data");
            ByteDanceVideoInfo videoInfo = new ByteDanceVideoInfo(data, token);
            videoInfoMapper.insert(videoInfo);
            imageIds.add(videoInfo.getId());
        }
        ResultMapUtils.setResultMap(resultMap, StatusCode.COMMON_SUCCESS.getCode());
        resultMap.put("imageIds", imageIds);
        return resultMap;
    }

    @Autowired
    private ByteDanceVideoInfoMapper videoInfoMapper;
    @Autowired
    private ByteDanceImageInfoMapper imageInfoMapper;

    @Override
    public Map<String, Object> uploadImageToBytedance(String accountId, String imageUrl) {
        Map<String, Object> resultMap = new HashMap<>();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject resultObject = uploadAdImage(token.getAccessToken(), imageUrl, token.getAccountId() + "");
        Integer code = resultObject.getInteger("code");
        String message = resultObject.getString("message");
        if (null == code || code != 0) {
            logger.info("图片文件上传失败,accountId:{},message:{}", accountId, message);
            ResultMapUtils.setResultMap(resultMap, StatusCode.BYTEDANCE_IMAGE_UPLOAD_FAIL.getCode());
            return resultMap;
        }
        JSONObject data = resultObject.getJSONObject("data");
        ByteDanceImageInfo imageInfo = new ByteDanceImageInfo(data, token);
        imageInfoService.saveOrUpdate(imageInfo);
        ResultMapUtils.setResultMap(resultMap, StatusCode.COMMON_SUCCESS.getCode());
        JSONArray array = new JSONArray();
        array.add(imageInfo.getId());
        resultMap.put("imageIds", array);
        return resultMap;
    }

    @Autowired
    private IByteDanceImageInfoService imageInfoService;

    @Override
    public Map<String, Object> getIndustryList(String accountId, Integer level) {
        Map<String, Object> resultMap = new HashMap<>();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject result = searchIndustry(token, level);
        JSONArray list = result.getJSONObject("data").getJSONArray("list");
        for (int i = 0; i < list.size(); i++) {
            JSONObject object = list.getJSONObject(i);
            SysCategory category = new SysCategory();
            String firstCode = object.getInteger("second_industry_id") + "";
            QueryWrapper<SysCategory> wrapper = new QueryWrapper();
            wrapper.eq("code", firstCode);
            SysCategory parent = sysCategoryMapper.selectOne(wrapper);
            category.setPid(parent.getId());
            category.setName(object.getString("third_industry_name"));
            category.setCode(object.getInteger("third_industry_id") + "");
            category.setCreateBy("admin");
            category.setCreateTime(new Date());
            category.setSysOrgCode("A01");
            sysCategoryMapper.insert(category);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "数据获取成功");
        resultMap.put("data", result);
        return resultMap;
    }

    @Override
    public JSONArray getByteDanceIndustryList(HttpServletRequest req) {
        JSONArray result = new JSONArray();
        String pid = "6c6b0ad4f49b65f3943528e8a35d74fc";
        List<SysCategory> categories = categoryService.getListByPid(pid);
        for (SysCategory category : categories) {
            JSONObject object = new JSONObject();
            object.put("value", category.getCode());
            object.put("label", category.getName());
            JSONArray childListArray = new JSONArray();
            List<SysCategory> childList = categoryService.getListByPid(category.getId());
            for (SysCategory child : childList) {
                JSONObject childObject = new JSONObject();
                childObject.put("value", child.getCode());
                childObject.put("label", child.getName());
                JSONArray secendChildListArray = new JSONArray();
                List<SysCategory> secendChildList = categoryService.getListByPid(child.getId());
                for (SysCategory secendChild : secendChildList) {
                    JSONObject secendChildObject = new JSONObject();
                    secendChildObject.put("value", secendChild.getCode());
                    secendChildObject.put("label", secendChild.getName());
                    secendChildListArray.add(secendChildObject);
                }
                childObject.put("children", secendChildListArray);
                childListArray.add(childObject);
            }
            object.put("children", childListArray);
            result.add(object);
        }
        return result;
    }


    @Autowired
    private ISysCategoryService categoryService;

    @Autowired
    private SysCategoryMapper sysCategoryMapper;

    public JSONObject searchIndustry(CTopOauthToken token, Integer level) {
        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/tools/industry/get/";
        if (null != level) {
            url += "?level=" + level;
        }
        // 构造请求
        HttpEntityEnclosingRequestBase httpEntity = new HttpEntityEnclosingRequestBase() {
            @Override
            public String getMethod() {
                return "GET";
            }
        };
        httpEntity.setHeader("Access-Token", token.getAccessToken());
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setURI(URI.create(url));
            httpEntity.setEntity(new StringEntity(new JSONObject().toJSONString(), ContentType.APPLICATION_JSON));
            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public JSONObject uploadAdImage(String accessToken, String imageUrl, String advertiserId) {
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_file_image_ad");
        // 构造请求
        HttpPost httpPost = new HttpPost(url);
        httpPost.setHeader("Access-Token", accessToken);
        MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create();
        // 其他参数
        entityBuilder.addTextBody("advertiser_id", advertiserId);
        entityBuilder.addTextBody("upload_type", BytedanceInterfaceConstant.UPLOAD_TYPE_BY_URL);
        entityBuilder.addTextBody("image_url", "https:" + imageUrl);

        HttpEntity entity = entityBuilder.build();
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpPost.setURI(URI.create(url));
            httpPost.setEntity(entity);
            response = client.execute(httpPost);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public JSONObject uploadAdvideo(String accessToken, String videoUrl, String advertiserId) {
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_file_video_ad");
        // 构造请求
        HttpPost httpPost = new HttpPost(url);
        httpPost.setHeader("Access-Token", accessToken);
        // 文件参数
        try {
            videoUrl = "https:" + videoUrl;
            URI uri = new URI(videoUrl);
            FileBody file = new FileBody(new File(uri));
            MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create().addPart("video_file", file);
            // 其他参数
            entityBuilder.addTextBody("advertiser_id", advertiserId);
            //TODO 需要后期优化修改代码逻辑
            entityBuilder.addTextBody("image_signature", DigestUtils.md5Hex(new FileInputStream(new File(uri))));
            HttpEntity entity = entityBuilder.build();
            client = HttpClientBuilder.create().build();
            httpPost.setURI(URI.create(url));
            httpPost.setEntity(entity);
            response = client.execute(httpPost);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }
    @Autowired
    private ICTopOauthTokenService tokenService;
}
